# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['model_subscription']

package_data = \
{'': ['*']}

install_requires = \
['django-lifecycle>=0.9.0,<0.10.0',
 'six>=1.14,<2.0',
 'typing_extensions>=3.7,<4.0']

extras_require = \
{':python_version >= "3.4" and python_version < "3.5"': ['typing>=3.6,<4.0']}

setup_kwargs = {
    'name': 'django-model-subscription',
    'version': '0.2.0',
    'description': 'Subscription model for a django model instance.',
    'long_description': '[![CircleCI](https://circleci.com/gh/tj-django/django-model-subscription.svg?style=shield)](https://circleci.com/gh/tj-django/django-model-subscription)\n[![Actions Status](https://github.com/jackton1/django-model-subscription/workflows/django%20model%20subscription%20test./badge.svg)](https://github.com/jackton1/django-model-subscription/actions?query=workflow%3A"django+model+subscription+test.")\n[![Documentation Status](https://readthedocs.org/projects/django-model-subscription/badge/?version=latest)](https://django-model-subscription.readthedocs.io/en/latest/?badge=latest)\n\n[![PyPI](https://img.shields.io/pypi/v/django-model-subscription)](https://pypi.org/project/django-model-subscription/) [![Total alerts](https://img.shields.io/lgtm/alerts/g/tj-django/django-model-subscription.svg?logo=lgtm&logoWidth=18)](https://lgtm.com/projects/g/tj-django/django-model-subscription/alerts/) [![Language grade: Python](https://img.shields.io/lgtm/grade/python/g/tj-django/django-model-subscription.svg?logo=lgtm&logoWidth=18)](https://lgtm.com/projects/g/tj-django/django-model-subscription/context:python) [![PyPI - License](https://img.shields.io/pypi/l/django-model-subscription.svg)](https://github.com/jackton1/django-model-subscription/blob/master/LICENSE)\n\n[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/django-model-subscription.svg)](https://pypi.org/project/django-model-subscription)\n[![PyPI - Django Version](https://img.shields.io/pypi/djversions/django-model-subscription.svg)](https://docs.djangoproject.com/en/2.2/releases/)\n[![Downloads](https://pepy.tech/badge/django-model-subscription)](https://pepy.tech/project/django-model-subscription)\n\n# [django-model-subscription](https://django-model-subscription.readthedocs.io/en/latest/installation.html)\n\n![Sreenshot](https://media.giphy.com/media/IgvrR33L6S7nFgH1by/giphy.gif)\n\n\n## Table of contents\n* [Motivation](#Motivation)\n* [Installation](#Installation)\n* [Usage](#Usage)\n* [Decorators](#Decorators)\n* [Setup Subscribers using AppConfig.ready](#setup-subscribers-using-appconfigready-recomended)\n* [Setup Subscribers with auto discovery](#setup-subscribers-using-auto-discovery)\n\n\n### Motivation\n\n- Using Observer Pattern notify subscribers about changes to a django model.\n- Decouple Business logic from Models.save\n- Support for bulk actions (Not available using django signals.)\n- Use noop subscribers when `settings.SUBSCRIPTION_RUN_EXTERNAL` is `False` \n  which prevents having to mock subscribers that call external services in testing, local development \n  environments.\n- Show changes to the instance after it has been updated i.e diff\'s the initial state and the \ncurrent state.\n\n![Screenshot](Subscriber.png)\n\n\n### Installation\n\n```bash\n$ pip install django-model-subscription\n```\n\nAdd `model_subscription` to your INSTALLED_APPS\n\n```python\nINSTALLED_APPS = [\n    ...,\n    \'model_subscription\',\n    ...\n]\n```\n\n\n\n\n### Usage\n\n##### Using the `SubscriptionModelMixin` and `SubscriptionQuerySet`\n\n```py\nfrom model_subscription.mixin import SubscriptionModelMixin\nfrom model_subscription.model import SubscriptionQuerySet\n\n\nclass TestModel(SubscriptionModelMixin, models.Model):\n    name = models.CharField(max_length=255)\n    \n    objects = SubscriptionQuerySet.as_manager()\n```\n\n##### Subclassing the `SubscriptionModel` base class.\n\n```py\nfrom model_subscription.model import SubscriptionModel\n\n\nclass TestModel(SubscriptionModel):\n    name = models.CharField(max_length=255)\n\n```\n\n\n#### Creating subscribers.\n\n- Using `OperationType`\n\n```python\nimport logging\nfrom model_subscription.decorators import subscribe\nfrom model_subscription.constants import OperationType\n\nlog = logging.getLogger(__name__)\n\n@subscribe(OperationType.CREATE, TestModel)\ndef handle_create(instance):\n    log.debug(\'Created {}\'.format(instance.name))\n\n\n```\n\n- Using `create_subscription` directly (succinct version).\n\n```python\n\nimport logging\nfrom model_subscription.decorators import create_subscription\n\nlog = logging.getLogger(__name__)\n\n@create_subscription(TestModel)\ndef handle_create(instance):\n    log.debug(\'Created {}\'.format(instance.name))\n\n\n```\n\n\n### Decorators\n\n* `subscribe`: Explicit (Requires a valid OperationType).\n\n\n#### (Create, Update, Delete) operations. \n\n* `create_subscription`: Subscribes to create operation i.e a new instance.\n\n```python\n@create_subscription(TestModel)\ndef handle_create(instance):\n    log.debug(\'1. Created {}\'.format(instance.name))\n```\n\n* `update_subscription`: Subscribes to updates also includes (`changed_data`).\n```python\n@update_subscription(TestModel)\ndef handle_update(instance, changed_data):\n    log.debug(\'Updated {} {}\'.format(instance.name, changed_data))\n```\n\n\n* `delete_subscription`: Subscribes to delete operation: \n\n> NOTE: The instance.pk is already set to None.\n\n```python\n@delete_subscription(TestModel)\ndef handle_delete(instance):\n    log.debug(\'Deleted {}\'.format(instance.name))\n```\n\n#### (Bulk Create, Bulk Update, Bulk Delete) operations. \n\n* `bulk_create_subscription`: Subscribe to bulk create operations.\n\n```python\n\n@bulk_create_subscription(TestModel)\ndef handle_bulk_create(instances):\n    for instance in instances:\n        log.debug(\'Bulk Created {}\'.format(instance.name))\n\n```\n\n\n* `bulk_update_subscription`: Subscribe to bulk update operations.\n\n```python\n@bulk_update_subscription(TestModel)\ndef handle_bulk_update(instances):\n    for instance in instances:\n        log.debug(\'Updated {}\'.format(instance.name))\n```\n\n\n* `bulk_delete_subscription`: Subscribe to bulk delete operations.\n\n```python\n\n@bulk_delete_subscription(TestModel)\ndef handle_bulk_delete(instances):\n    for instance in instances:\n        log.debug(\'Deleted {}\'.format(instance.name))\n\n```\n\n\n### Setup Subscribers using AppConfig.ready `(Recomended)`. \n\n\nUpdate you `apps.py`\n\n\n```python\n\nfrom django.apps import AppConfig\n\n\nclass MyAppConfig(AppConfig):\n    name = \'myapp\'\n\n    def ready(self):\n        from myapp import subscriptions\n\n```\n\n\n\n### Setup Subscribers using auto discovery.\n\nBy default the `settings.SUBSCRIPTION_AUTO_DISCOVER` is set to `False`.\n\nTo use auto discovery this is not recommended as it would notify the subscribers \nwherever the model is used i.e IPython notebook, external scripts.\n\nIn your `settings.py` add\n\n```python\n\nSUBSCRIPTION_AUTO_DISCOVER = True\n\n```\n\n#### Setting up the `SUBSCRIPTION_MODULE`\n\n> NOTE: This is only required when ``SUBSCRIPTION_AUTO_DISCOVER = True``\n\n```python\n\nSUBSCRIPTION_MODULE  = \'subscription\' \n\n```\n\n#### Resources\n- https://python-3-patterns-idioms-test.readthedocs.io/en/latest/Observer.html\n- https://refactoring.guru/design-patterns/observer\n- https://hackernoon.com/observer-vs-pub-sub-pattern-50d3b27f838c\n\nTODO\'s\n- Supporting field level subscriptions.\n- Support class based subscribers which implements `__call__`\n- Extend to include custom OperationType.\n- Add support for using a single class to manage multiple actions i.e MyClass.update, MyClass.create.\n',
    'author': 'Tonye Jack',
    'author_email': 'tonyejck@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://django-model-subscription.readthedocs.io/en/latest/index.html',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6.0,<3.10.0',
}


setup(**setup_kwargs)
